==> snapconnect_futures-4.0.1a3-py2-none-any.whl: snapconnect_futures/serial_manager.py <==
"""Manage serial connections with asynchronous Futures."""
import logging
import binascii

from datetime import timedelta

from tornado.gen import coroutine, Future, Return, with_timeout

from snapconnect import snap

log = logging.getLogger('snapconnect_futures.serial')


class FutureSerialManager(object):
    def __init__(self, snapconnect):
        """Wrap a SNAP Connect instance with the ability to perform async serial operations with Futures

        :param snap.Snap snapconnect: SNAP Connect instance to wrap.
        """
        self._snap = snapconnect

        self._serial_futures = {}  # (serial_type, serial_port) -> Future
        self._snap.set_hook(snap.hooks.HOOK_SERIAL_OPEN, self._on_serial_open)
        self._snap.set_hook(snap.hooks.HOOK_SERIAL_CLOSE, self._on_serial_close)

    @coroutine
    def serial_operation(self, is_open, serial_type, port, forward_groups=None, baudrate=None, timeout=2):
        """Open or close a serial port and prepare to resolve the future

        If the serial connection fails to open an exception is raised

        :param bool is_open: If True, this is an open operation. If False this is a close
        :param int serial_type: Serial type (Ex: SERIAL_TYPE_SNAPSTICK100, SERIAL_TYPE_SNAPSTICK200, SERIAL_TYPE_RS232)
        :param int|str port: Port to connect to for the given serial type
        :param int|None forward_groups: Multi-cast groups to forward through. None to use the DV param defaults.
        :param int|None baudrate: The baudrate for the serial connection. None to use the SNAP Connect default.
        :param float timeout: Time in seconds before an open has failed

        Note: forward_groups and baudrate are unused by

        Raises a TimeoutError if we don't open in time
        """
        if (serial_type, port) in self._serial_futures:
            raise RuntimeError("Already attempting a serial operation")

        f = Future()

        self._serial_futures[(serial_type, port)] = f

        try:
            if is_open:
                log.info("Attempting to open a serial connection with baudrate %r", baudrate)
                self._snap.open_serial(serial_type, port, forward_groups=forward_groups, baudrate=baudrate)
            else:
                log.info("Attempting to close a serial connection")
                self._snap.close_serial(serial_type, port)

            result = yield with_timeout(timedelta(seconds=timeout), f)
        except:
            self._serial_futures.pop((serial_type, port), None)
            raise

        raise Return(result)

    def _on_serial_close(self, serial_type, port):
        """Display logging information when the serial bridge connection is closed

        This is a callback for the snap.hooks.HOOK_SERIAL_CLOSE hook

        :param int serial_type: Enumeration representing the serial type
        :param str|int port: The port passed to open_serial
        """
        log.info("Bridge connection closed")

        serial_future = self._serial_futures.pop((serial_type, port), None)

        if serial_future is None:
            log.error("Unexpected serial close call for type:%s port:%s", serial_type, port)
            return
        else:
            serial_future.set_result(True)

    def _on_serial_open(self, serial_type, port, addr=None):
        """Display logging information when the serial bridge connection is opened

        This is a callback for the snap.hooks.HOOK_SERIAL_OPEN hook

        :param int serial_type: Enumeration representing the serial type
        :param str|int port: The port passed to open_serial
        :param str|None addr: SNAP Address of the serial bridge (hex) or None if the connection failed
        """
        serial_future = self._serial_futures.pop((serial_type, port), None)

        if serial_future is None:
            log.error("Unexpected serial open call for type:%s port:%s addr:%r", serial_type, port, addr)
            return

        if addr:
            addr = binascii.hexlify(addr)
            log.info("Serial connection opened to %s", addr)
            serial_future.set_result(addr)
        else:
            log.critical("Unable to open serial connection")
            serial_future.set_result(None)

==> snapconnect_futures-7.1.0-py2-none-any.whl: snapconnect_futures/serial_manager.py <==
"""Manage serial connections with asynchronous Futures."""
import logging
import binascii

from datetime import timedelta

from tornado.gen import coroutine, Future, Return, with_timeout

from snapconnect import snap

log = logging.getLogger('snapconnect_futures.serial')


class FutureSerialManager(object):
    def __init__(self, snapconnect):
        """Wrap a SNAP Connect instance with the ability to perform async serial operations with Futures

        Args:
            snapconnect (snap.Snap): SNAP Connect instance to wrap.
        """
        self._snap = snapconnect

        self._serial_futures = {}  # (serial_type, serial_port) -> Future
        self._snap.set_hook(snap.hooks.HOOK_SERIAL_OPEN, self._on_serial_open)
        self._snap.set_hook(snap.hooks.HOOK_SERIAL_CLOSE, self._on_serial_close)

    @coroutine
    def serial_operation(self, is_open, serial_type, port, forward_groups=None, baudrate=None, timeout=2):
        """Open or close a serial port and prepare to resolve the future

        If the serial connection fails to open an exception is raised

        Args:
            is_open (bool): If True, this is an open operation. If False this is a close
            serial_type (int): Serial type (Ex: SERIAL_TYPE_SNAPSTICK100, SERIAL_TYPE_SNAPSTICK200, SERIAL_TYPE_RS232)
            port (int | str): Port to connect to for the given serial type
            forward_groups (int | None): Multi-cast groups to forward through. None to use the DV param defaults.
            baudrate (int | None): The baudrate for the serial connection. None to use the SNAP Connect default.
            timeout (float): Time in seconds before an open has failed

        Notes:
            forward_groups and baudrate are unused by

        Raises:
            TimeoutError: If we don't open in time
        """
        if (serial_type, port) in self._serial_futures:
            raise RuntimeError("Already attempting a serial operation")

        f = Future()

        self._serial_futures[(serial_type, port)] = f

        try:
            if is_open:
                log.info("Attempting to open a serial connection with baudrate %r", baudrate)
                self._snap.open_serial(serial_type, port, forward_groups=forward_groups, baudrate=baudrate)
            else:
                log.info("Attempting to close a serial connection")
                self._snap.close_serial(serial_type, port)

            result = yield with_timeout(timedelta(seconds=timeout), f)
        except:
            self._serial_futures.pop((serial_type, port), None)
            raise

        raise Return(result)

    def _on_serial_close(self, serial_type, port):
        """Display logging information when the serial bridge connection is closed

        This is a callback for the snap.hooks.HOOK_SERIAL_CLOSE hook

        Args:
            serial_type (int): Enumeration representing the serial type
            port (int | str): The port passed to open_serial
        """
        log.info("Bridge connection closed")

        serial_future = self._serial_futures.pop((serial_type, port), None)

        if serial_future is None:
            log.error("Unexpected serial close call for type:%s port:%s", serial_type, port)
            return
        else:
            serial_future.set_result(True)

    def _on_serial_open(self, serial_type, port, addr=None):
        """Display logging information when the serial bridge connection is opened

        This is a callback for the snap.hooks.HOOK_SERIAL_OPEN hook

        Args:
            serial_type (int): Enumeration representing the serial type
            port (str | int): The port passed to open_serial
            addr (str | None): SNAP Address of the serial bridge (hex) or None if the connection failed
        """
        serial_future = self._serial_futures.pop((serial_type, port), None)

        if serial_future is None:
            log.error("Unexpected serial open call for type:%s port:%s addr:%r", serial_type, port, addr)
            return

        if addr:
            addr = binascii.hexlify(addr)
            log.info("Serial connection opened to %s", addr)
            serial_future.set_result(addr)
        else:
            log.critical("Unable to open serial connection")
            serial_future.set_result(None)
